@@ -2,18 +2,122 @@
 from __future__ import division
 import colorsys
 import numpy as np
+from scipy import stats
 import pandas as pd
 import statsmodels.api as sm
 import matplotlib as mpl
 import matplotlib.pyplot as plt
-from six import string_types
 from six.moves import range
+import warnings
 import moss
 
 from seaborn.utils import (color_palette, husl_palette, blend_palette,
                            desaturate, _kde_support)
 
 
+def _box_reshape(vals, groupby, names, order):
+    """Reshape the box/violinplot input options and find plot labels."""
+
+    # Set up default label outputs
+    xlabel, ylabel = None, None
+
+    # If order is provided, make sure it was used correctly
+    if order is not None:
+        # Assure that order is the same length as names, if provided
+        if names is not None:
+            if len(order) != len(names):
+                raise ValueError("`order` must have same length as `names`")
+        # Assure that order is only used with the right inputs
+        is_pd = isinstance(vals, pd.Series) or isinstance(vals, pd.DataFrame)
+        if not is_pd:
+            raise ValueError("`vals` must be a Pandas object to use `order`.")
+
+    # Handle case where data is a wide DataFrame
+    if isinstance(vals, pd.DataFrame):
+        if order is not None:
+            vals = vals[order]
+        if names is None:
+            names = vals.columns.tolist()
+        if vals.columns.name is not None:
+            xlabel = vals.columns.name
+        vals = vals.values.T
+
+    # Handle case where data is a long Series and there is a grouping object
+    elif isinstance(vals, pd.Series) and groupby is not None:
+        groups = pd.groupby(vals, groupby).groups
+        order = sorted(groups) if order is None else order
+        if hasattr(groupby, "name"):
+            if groupby.name is not None:
+                xlabel = groupby.name
+        if vals.name is not None:
+            ylabel = vals.name
+        vals = [vals.reindex(groups[name]) for name in order]
+        if names is None:
+            names = order
+
+    else:
+
+        # Handle case where the input data is an array or there was no groupby
+        if hasattr(vals, 'shape'):
+            if len(vals.shape) == 1:
+                if np.isscalar(vals[0]):
+                    vals = [vals]
+                else:
+                    vals = list(vals)
+            elif len(vals.shape) == 2:
+                nr, nc = vals.shape
+                if nr == 1:
+                    vals = [vals]
+                elif nc == 1:
+                    vals = [vals.ravel()]
+                else:
+                    vals = [vals[:, i] for i in range(nc)]
+            else:
+                error = "Input `vals` can have no more than 2 dimensions"
+                raise ValueError(error)
+
+        # This should catch things like flat lists
+        elif np.isscalar(vals[0]):
+            vals = [vals]
+
+        # By default, just use the plot positions as names
+        if names is None:
+            names = list(range(1, len(vals) + 1))
+        elif hasattr(names, "name"):
+            if names.name is not None:
+                xlabel = names.name
+
+    # Now convert vals to a common representation
+    # The plotting functions will work with a list of arrays
+    # The list allows each array to possibly be of a different length
+    vals = [np.asarray(a, np.float) for a in vals]
+
+    return vals, xlabel, ylabel, names
+
+
+def _box_colors(vals, color):
+    """Find colors to use for boxplots or violinplots."""
+    if color is None:
+        colors = husl_palette(len(vals), l=.7)
+    else:
+        try:
+            color = mpl.colors.colorConverter.to_rgb(color)
+            colors = [color for _ in vals]
+        except ValueError:
+                colors = color_palette(color, len(vals))
+
+    # Desaturate a bit because these are patches
+    colors = [mpl.colors.colorConverter.to_rgb(c) for c in colors]
+    colors = [desaturate(c, .7) for c in colors]
+
+    # Determine the gray color for the lines
+    light_vals = [colorsys.rgb_to_hls(*c)[1] for c in colors]
+    l = min(light_vals) * .6
+    gray = (l, l, l)
+
+    return colors, gray
+
+
 def boxplot(vals, groupby=None, names=None, join_rm=False, order=None,
             color=None, alpha=None, fliersize=3, linewidth=1.5, widths=.8,
             ax=None, **kwargs):
@@ -21,17 +125,18 @@ def boxplot(vals, groupby=None, names=None, join_rm=False, order=None,
 
     Parameters
     ----------
-    vals : DataFrame, sequence of vectors, or Series.
-        Data for plot. DataFrames are assuemd to be "wide" with each column
-        mapping to a box. Other two-dimensional data is assumed to be a
-        sequence where each item is the data that will go into a box. Can
-        also provide one long Series in conjunction with a grouping element
-        as the `groupy` parameter.
+    vals : DataFrame, Series, 2D array, list of vectors, or vector.
+        Data for plot. DataFrames and 2D arrays are assuemd to be "wide" with
+        each column mapping to a box. Lists of data are assumed to have one
+        element per box.  Can also provide one long Series in conjunction with
+        a grouping element as the `groupy` parameter to reshape the data into
+        several boxes. Otherwise 1D data will produce a single box.
     groupby : grouping object
         If `vals` is a Series, this is used to group into boxes by calling
         pd.groupby(vals, groupby).
     names : list of strings, optional
-        Names to plot on x axis, otherwise plots numbers.
+        Names to plot on x axis; otherwise plots numbers. This will override
+        names inferred from Pandas inputs.
     order : list of strings, optional
         If vals is a Pandas object with name information, you can control the
         order of the boxes by providing the box names in your preferred order.
@@ -59,63 +164,14 @@ def boxplot(vals, groupby=None, names=None, join_rm=False, order=None,
     if ax is None:
         ax = plt.gca()
 
-    # Handle case where data is a wide DataFrame
-    if isinstance(vals, pd.DataFrame):
-        if names is None:
-            names = vals.columns
-        if vals.columns.name is not None:
-            xlabel = vals.columns.name
-        else:
-            xlabel = None
-        if order is not None:
-            vals = vals[order]
-            names = order
-        vals = vals.values
-        ylabel = None
-
-    # Handle case where data is a long Series and there is a grouping object
-    elif isinstance(vals, pd.Series) and groupby is not None:
-        if names is None:
-            names = np.sort(pd.unique(groupby))
-        if hasattr(groupby, "name"):
-            xlabel = groupby.name
-        ylabel = vals.name
-        grouped_vals = pd.groupby(vals, groupby).values
-        if order is not None:
-            grouped_vals = grouped_vals[order]
-            names = order
-        vals = grouped_vals.values
-    else:
-        xlabel = None
-        ylabel = None
+    # Reshape and find labels for the plot
+    vals, xlabel, ylabel, names = _box_reshape(vals, groupby, names, order)
 
     # Draw the boxplot using matplotlib
     boxes = ax.boxplot(vals, patch_artist=True, widths=widths, **kwargs)
-    vals = np.atleast_2d(vals).T
 
-    # Sort out the inner box color
-    if color is None:
-        colors = husl_palette(len(vals), l=.7)
-    else:
-        color_is_color = (not isinstance(color, string_types)
-                          and not isinstance(color, tuple))
-        if color_is_color:
-            colors = color
-        else:
-            try:
-                color = mpl.colors.colorConverter.to_rgb(color)
-                colors = [color for _ in vals]
-            except ValueError:
-                colors = color_palette(color, len(vals))
-
-    # Desaturate a bit because these are patches
-    colors = [mpl.colors.colorConverter.to_rgb(c) for c in colors]
-    colors = [desaturate(c, .7) for c in colors]
-
-    # Determine the gray color for the lines
-    light_vals = [colorsys.rgb_to_hls(*c)[1] for c in colors]
-    l = min(light_vals) * .6
-    gray = (l, l, l)
+    # Find plot colors
+    colors, gray = _box_colors(vals, color)
 
     # Set the new aesthetics
     for i, box in enumerate(boxes["boxes"]):
@@ -145,19 +201,17 @@ def boxplot(vals, groupby=None, names=None, join_rm=False, order=None,
 
     # Draw the joined repeated measures
     if join_rm:
-        x, y = np.arange(1, len(vals.T) + 1), vals.T
+        x, y = np.arange(1, len(np.transpose(vals)) + 1), np.transpose(vals)
         if not vertical:
             x, y = y, x
         ax.plot(x, y, color=gray, alpha=2. / 3)
 
     # Label the axes and ticks
-    if not vertical:
+    if vertical:
+        ax.set_xticklabels(names)
+    else:
+        ax.set_yticklabels(names)
         xlabel, ylabel = ylabel, xlabel
-    if names is not None:
-        if vertical:
-            ax.set_xticklabels(names)
-        else:
-            ax.set_yticklabels(names)
     if xlabel is not None:
         ax.set_xlabel(xlabel)
     if ylabel is not None:
@@ -174,7 +228,6 @@ def boxplot(vals, groupby=None, names=None, join_rm=False, order=None,
 
 def violin(*args, **kwargs):
     """Deprecated old name for violinplot. Please update your code."""
-    import warnings
     warnings.warn("violin() is deprecated; please use violinplot()",
                   UserWarning)
     return violinplot(*args, **kwargs)
@@ -189,12 +242,12 @@ def violinplot(vals, groupby=None, inner="box", color=None, positions=None,
 
     Parameters
     ----------
-    vals : DataFrame, sequence of vectors, or Series.
-        Data for plot. DataFrames are assuemd to be "wide" with each column
-        mapping to a box. Other two-dimensional data is assumed to be a
-        sequence where each item is the data that will go into a box. Can
-        also provide one long Series in conjunction with a grouping element
-        as the `groupy` parameter.
+    vals : DataFrame, Series, 2D array, or list of vectors.
+        Data for plot. DataFrames and 2D arrays are assuemd to be "wide" with
+        each column mapping to a box. Lists of data are assumed to have one
+        element per box.  Can also provide one long Series in conjunction with
+        a grouping element as the `groupy` parameter to reshape the data into
+        several violins. Otherwise 1D data will produce a single violins.
     groupby : grouping object
         If `vals` is a Series, this is used to group into boxes by calling
         pd.groupby(vals, groupby).
@@ -205,10 +258,12 @@ def violinplot(vals, groupby=None, inner="box", color=None, positions=None,
     positions : number or sequence of numbers
         Position of first violin or positions of each violin.
     names : list of strings, optional
-        Names to plot on x axis; otherwise plots numbers.
+        Names to plot on x axis; otherwise plots numbers. This will override
+        names inferred from Pandas inputs.
     order : list of strings, optional
         If vals is a Pandas object with name information, you can control the
-        order of the plot by providing the violin names in your preferred order.
+        order of the plot by providing the violin names in your preferred
+        order.
     kernel : {'gau' | 'cos' | 'biw' | 'epa' | 'tri' | 'triw' }
         Code for shape of kernel to fit with.
     bw : {'scott' | 'silverman' | scalar}
@@ -240,81 +295,11 @@ def violinplot(vals, groupby=None, inner="box", color=None, positions=None,
     if ax is None:
         ax = plt.gca()
 
-    # Find existing names
-    if isinstance(vals, pd.DataFrame):
-        if names is None:
-            names = vals.columns
-        if vals.columns.name is not None:
-            xlabel = vals.columns.name
-        else:
-            xlabel = None
-        ylabel = None
-        if order is not None:
-            vals = vals[order]
-            names = order
-        vals = vals.values
-
-    # Possibly perform a group-by to get the batches
-    elif isinstance(vals, pd.Series) and groupby is not None:
-        if hasattr(groupby, "name"):
-            xlabel = groupby.name
-        if names is None:
-            names = np.sort(pd.unique(groupby))
-        ylabel = vals.name
-        grouped_vals = pd.groupby(vals, groupby).values
-        if order is not None:
-            grouped_vals = grouped_vals[order]
-            names = order
-        vals = grouped_vals.values
-    else:
-        xlabel = None
-        ylabel = None
-
-    # Handle the input data (from pyplot.boxplot)
-    if hasattr(vals, 'shape'):
-        if len(vals.shape) == 1:
-            if hasattr(vals[0], 'shape'):
-                vals = list(vals)
-            else:
-                vals = [vals]
-        elif len(vals.shape) == 2:
-            nr, nc = vals.shape
-            if nr == 1:
-                vals = [vals]
-            elif nc == 1:
-                vals = [vals.ravel()]
-            else:
-                vals = [vals[:, i] for i in range(nc)]
-        else:
-            raise ValueError("Input x can have no more than 2 dimensions")
-    if not hasattr(vals[0], '__len__'):
-        vals = [vals]
-
-    vals = [np.asarray(a, float) for a in vals]
+    # Reshape and find labels for the plot
+    vals, xlabel, ylabel, names = _box_reshape(vals, groupby, names, order)
 
     # Sort out the plot colors
-    if color is None:
-        colors = husl_palette(len(vals), l=.7)
-    else:
-        color_is_color = (not isinstance(color, string_types)
-                          and not isinstance(color, tuple))
-        if color_is_color:
-            colors = color
-        else:
-            try:
-                color = mpl.colors.colorConverter.to_rgb(color)
-                colors = [color for _ in vals]
-            except ValueError:
-                colors = color_palette(color, len(vals))
-
-    # Use somewhat desaturated colors because we're drawing patches
-    colors = [mpl.colors.colorConverter.to_rgb(c) for c in colors]
-    colors = [desaturate(c, .7) for c in colors]
-
-    # Find the shade of gray for lines
-    light_vals = [colorsys.rgb_to_hls(*c)[1] for c in colors]
-    l = min(light_vals) * .6
-    gray = (l, l, l)
+    colors, gray = _box_colors(vals, color)
 
     # Initialize the kwarg dict for the inner plot
     if inner_kws is None:
@@ -396,6 +381,14 @@ def violinplot(vals, groupby=None, inner="box", color=None, positions=None,
     return ax
 
 
+def _freedman_diaconis_bins(a):
+    """Calculate number of hist bins using Freedman-Diaconis rule."""
+    # From http://stats.stackexchange.com/questions/798/
+    a = np.asarray(a)
+    h = 2 * moss.iqr(a) / (len(a) ** (1 / 3))
+    return np.ceil((a.max() - a.min()) / h)
+
+
 def distplot(a, bins=None, hist=True, kde=True, rug=False, fit=None,
              hist_kws=None, kde_kws=None, rug_kws=None, fit_kws=None,
              color=None, vertical=False, axlabel=None, ax=None):
@@ -467,9 +460,7 @@ def distplot(a, bins=None, hist=True, kde=True, rug=False, fit=None,
 
     if hist:
         if bins is None:
-            # From http://stats.stackexchange.com/questions/798/
-            h = 2 * moss.iqr(a) * len(a) ** -(1 / 3)
-            bins = (a.max() - a.min()) / h
+            bins = _freedman_diaconis_bins(a)
         hist_alpha = hist_kws.pop("alpha", 0.4)
         orientation = "horizontal" if vertical else "vertical"
         hist_color = hist_kws.pop("color", color)
@@ -508,8 +499,8 @@ def distplot(a, bins=None, hist=True, kde=True, rug=False, fit=None,
     return ax
 
 
-def _univariate_kde(data, shade, vertical, kernel, bw, gridsize, cut,
-                    clip, legend, ax, **kwargs):
+def _univariate_kdeplot(data, shade, vertical, kernel, bw, gridsize, cut,
+                        clip, legend, ax, **kwargs):
     """Plot a univariate kernel density estimate on one of the axes."""
 
     # Sort out the clipping
@@ -517,10 +508,22 @@ def _univariate_kde(data, shade, vertical, kernel, bw, gridsize, cut,
         clip = (-np.inf, np.inf)
 
     # Calculate the KDE
-    fft = kernel == "gau"
-    kde = sm.nonparametric.KDEUnivariate(data)
-    kde.fit(kernel, bw, fft, gridsize=gridsize, cut=cut, clip=clip)
-    x, y = kde.support, kde.density
+    try:
+        # Prefer using statsmodels for kernel flexibility
+        x, y = _statsmodels_univariate_kde(data, kernel, bw,
+                                           gridsize, cut, clip)
+    except ImportError:
+        # Fall back to scipy if missing statsmodels
+        if kernel != "gau":
+            kernel = "gau"
+            msg = "Kernel other than `gau` requires statsmodels."
+            warnings.warn(msg, UserWarning)
+        x, y = _scipy_univariate_kde(data, bw, gridsize, cut, clip)
+
+    # Make sure the density is nonnegative
+    y = np.amax(np.c_[np.zeros_like(y), y], axis=1)
+
+    # Flip the data if the plot should be on the y axis
     if vertical:
         x, y = y, x
 
@@ -554,8 +557,29 @@ def _univariate_kde(data, shade, vertical, kernel, bw, gridsize, cut,
     return ax
 
 
-def _bivariate_kde(x, y, filled, kernel, bw, gridsize, cut, clip, axlabel, ax,
-                   **kwargs):
+def _statsmodels_univariate_kde(data, kernel, bw, gridsize, cut, clip):
+    """Compute a univariate kernel density estimate using statsmodels."""
+    from statsmodels import nonparametric
+    fft = kernel == "gau"
+    kde = nonparametric.kde.KDEUnivariate(data)
+    kde.fit(kernel, bw, fft, gridsize=gridsize, cut=cut, clip=clip)
+    grid, y = kde.support, kde.density
+    return grid, y
+
+
+def _scipy_univariate_kde(data, bw, gridsize, cut, clip):
+    """Compute a univariate kernel density estimate using scipy."""
+    kde = stats.gaussian_kde(data, bw_method=bw)
+    if isinstance(bw, str):
+        bw = "scotts" if bw == "scott" else bw
+        bw = getattr(kde, "%s_factor" % bw)()
+    grid = _kde_support(data, bw, gridsize, cut, clip)
+    y = kde(grid)
+    return grid, y
+
+
+def _bivariate_kdeplot(x, y, filled, kernel, bw, gridsize, cut, clip, axlabel,
+                       ax, **kwargs):
     """Plot a joint KDE estimate as a bivariate contour plot."""
 
     # Determine the clipping
@@ -565,18 +589,10 @@ def _bivariate_kde(x, y, filled, kernel, bw, gridsize, cut, clip, axlabel, ax,
         clip = [clip, clip]
 
     # Calculate the KDE
-    if isinstance(bw, str):
-        bw_func = getattr(sm.nonparametric.bandwidths, "bw_" + bw)
-        x_bw = bw_func(x)
-        y_bw = bw_func(y)
-        bw = [x_bw, y_bw]
-    elif np.isscalar(bw):
-        bw = [bw, bw]
-    kde = sm.nonparametric.KDEMultivariate([x, y], "cc", bw)
-    x_support = _kde_support(x, kde.bw[0], gridsize, cut, clip[0])
-    y_support = _kde_support(y, kde.bw[1], gridsize, cut, clip[1])
-    xx, yy = np.meshgrid(x_support, y_support)
-    z = kde.pdf([xx.ravel(), yy.ravel()]).reshape(xx.shape)
+    try:
+        xx, yy, z = _statsmodels_bivariate_kde(x, y, bw, gridsize, cut, clip)
+    except ImportError:
+        xx, yy, z = _scipy_bivariate_kde(x, y, bw, gridsize, cut, clip)
 
     # Plot the contours
     n_levels = kwargs.pop("n_levels", 10)
@@ -598,6 +614,38 @@ def _bivariate_kde(x, y, filled, kernel, bw, gridsize, cut, clip, axlabel, ax,
     return ax
 
 
+def _statsmodels_bivariate_kde(x, y, bw, gridsize, cut, clip):
+    """Compute a bivariate kde using statsmodels."""
+    from statsmodels import nonparametric
+    if isinstance(bw, str):
+        bw_func = getattr(nonparametric.bandwidths, "bw_" + bw)
+        x_bw = bw_func(x)
+        y_bw = bw_func(y)
+        bw = [x_bw, y_bw]
+    elif np.isscalar(bw):
+        bw = [bw, bw]
+    kde = nonparametric.kernel_density.KDEMultivariate([x, y], "cc", bw)
+    x_support = _kde_support(x, kde.bw[0], gridsize, cut, clip[0])
+    y_support = _kde_support(y, kde.bw[1], gridsize, cut, clip[1])
+    xx, yy = np.meshgrid(x_support, y_support)
+    z = kde.pdf([xx.ravel(), yy.ravel()]).reshape(xx.shape)
+    return xx, yy, z
+
+
+def _scipy_bivariate_kde(x, y, bw, gridsize, cut, clip):
+    """Compute a bivariate kde using scipy."""
+    data = np.c_[x, y]
+    kde = stats.gaussian_kde(data.T)
+    if isinstance(bw, str):
+        bw = "scotts" if bw == "scott" else bw
+        bw = getattr(kde, "%s_factor" % bw)()
+    x_support = _kde_support(data[:, 0], bw, gridsize, cut, clip[0])
+    y_support = _kde_support(data[:, 1], bw, gridsize, cut, clip[1])
+    xx, yy = np.meshgrid(x_support, y_support)
+    z = kde([xx.ravel(), yy.ravel()]).reshape(xx.shape)
+    return xx, yy, z
+
+
 def kdeplot(data, data2=None, shade=False, vertical=False, kernel="gau",
             bw="scott", gridsize=100, cut=3, clip=None, legend=True, ax=None,
             **kwargs):
@@ -647,22 +695,22 @@ def kdeplot(data, data2=None, shade=False, vertical=False, kernel="gau",
     bivariate = False
     if isinstance(data, np.ndarray) and np.ndim(data) > 1:
         bivariate = True
-        x, y = data.T
+        x, y = data.astype(np.float64).T
     elif isinstance(data, pd.DataFrame) and np.ndim(data) > 1:
         bivariate = True
-        x = data.iloc[:, 0]
-        y = data.iloc[:, 1]
+        x = data.iloc[:, 0].values.astype(np.float64)
+        y = data.iloc[:, 1].values.astype(np.float64)
     elif data2 is not None:
         bivariate = True
         x = data
         y = data2
 
     if bivariate:
-        ax = _bivariate_kde(x, y, shade, kernel, bw, gridsize,
-                            cut, clip, legend, ax, **kwargs)
+        ax = _bivariate_kdeplot(x, y, shade, kernel, bw, gridsize,
+                                cut, clip, legend, ax, **kwargs)
     else:
-        ax = _univariate_kde(data, shade, vertical, kernel, bw,
-                             gridsize, cut, clip, legend, ax, **kwargs)
+        ax = _univariate_kdeplot(data, shade, vertical, kernel, bw,
+                                 gridsize, cut, clip, legend, ax, **kwargs)
 
     return ax
 
